@@ -19,14 +19,26 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
+#include "gitversion.h"
+#else
+#   ifndef MEDIASTREAMER_VERSION
+#   define MEDIASTREAMER_VERSION "unknown"
+#   endif
+#	ifndef GIT_VERSION
+#	define GIT_VERSION "unknown"
+#	endif
 #endif
 
+#include "mediastreamer2/mscommon.h"
+#include "mediastreamer2/mscodecutils.h"
+#include "mediastreamer2/msfilter.h"
+#include <ortp/ortp_srtp.h>
+
 extern void __register_ffmpeg_encoders_if_possible(void);
 extern void ms_ffmpeg_check_init();
-extern void libmsandroiddisplay_init(void);
-
-#include "mediastreamer2/mscommon.h"
-#include "mediastreamer2/msfilter.h"
+extern bool_t libmsandroiddisplay_init(void);
+extern void libmsandroiddisplaybad_init(void);
+extern void libmsandroidopengldisplay_init(void);
 
 #include "alldescs.h"
 #include "mediastreamer2/mssndcard.h"
@@ -50,9 +62,8 @@ extern void libmsandroiddisplay_init(void);
 #include <dlfcn.h>
 #endif
 
-#if defined(__APPLE__) && !defined(__GNUC__)
-#import <Cocoa/Cocoa.h>
-#include <Foundation/Foundation.h> 
+#ifdef __APPLE__
+   #include "TargetConditionals.h"
 #endif
 
 #ifdef ANDROID
@@ -63,6 +74,17 @@ extern void libmsandroiddisplay_init(void);
 static MSList *ms_plugins_loaded_list;
 #endif
 
+static unsigned int cpu_count = 1;
+
+unsigned int ms_get_cpu_count() {
+	return cpu_count;
+}
+
+void ms_set_cpu_count(unsigned int c) {
+	ms_message("CPU count set to %d", c);
+	cpu_count = c;
+}
+
 MSList *ms_list_new(void *data){
 	MSList *new_elem=(MSList *)ms_new(MSList,1);
 	new_elem->prev=new_elem->next=NULL;
@@ -258,21 +280,18 @@ MSList *ms_list_copy(const MSList *list){
 }
 
 
-#ifdef __APPLE__
-#define PLUGINS_EXT ".dylib"
-#else
-#define PLUGINS_EXT ".so"
+#ifndef PLUGINS_EXT
+	#define PLUGINS_EXT ".so"
 #endif
-
 typedef void (*init_func_t)(void);
 
 int ms_load_plugins(const char *dir){
 	int num=0;
 #if defined(WIN32) && !defined(_WIN32_WCE)
-	WIN32_FIND_DATA FileData; 
-	HANDLE hSearch; 
-	char szDirPath[1024]; 
-	char szPluginFile[1024]; 
+	WIN32_FIND_DATA FileData;
+	HANDLE hSearch;
+	char szDirPath[1024];
+	char szPluginFile[1024];
 	BOOL fFinished = FALSE;
 	const char *tmp=getenv("DEBUG");
 	BOOL debug=(tmp!=NULL && atoi(tmp)==1);
@@ -289,13 +308,13 @@ int ms_load_plugins(const char *dir){
 	}
 	snprintf(szDirPath, sizeof(szDirPath), "%s", dir);
 
-	while (!fFinished) 
+	while (!fFinished)
 	{
 		/* load library */
 		HINSTANCE os_handle;
 		UINT em;
 		if (!debug) em = SetErrorMode (SEM_FAILCRITICALERRORS);
-		
+
 		snprintf(szPluginFile, sizeof(szPluginFile), "%s\\%s", szDirPath, FileData.cFileName);
 		os_handle = LoadLibraryEx (szPluginFile, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
 		if (os_handle==NULL)
@@ -305,10 +324,10 @@ int ms_load_plugins(const char *dir){
 		}
 		if (!debug) SetErrorMode (em);
 		if (os_handle==NULL)
-			ms_error("Fail to load plugin %s", szPluginFile); 
+			ms_error("Fail to load plugin %s", szPluginFile);
 		else{
 			init_func_t initroutine;
-			char szPluginName[256]; 
+			char szPluginName[256];
 			char szMethodName[256];
 			char *minus;
 			snprintf(szPluginName, 256, "%s", FileData.cFileName);
@@ -330,16 +349,16 @@ int ms_load_plugins(const char *dir){
 				}
 		}
 		if (!FindNextFile(hSearch, &FileData)) {
-			if (GetLastError() == ERROR_NO_MORE_FILES){ 
-				fFinished = TRUE; 
-			} 
-			else 
-			{ 
-				ms_error("couldn't find next plugin dll."); 
-				fFinished = TRUE; 
-			} 
+			if (GetLastError() == ERROR_NO_MORE_FILES){
+				fFinished = TRUE;
+			}
+			else
+			{
+				ms_error("couldn't find next plugin dll.");
+				fFinished = TRUE;
+			}
 		}
-	} 
+	}
 	/* Close the search handle. */
 	FindClose(hSearch);
 
@@ -347,7 +366,7 @@ int ms_load_plugins(const char *dir){
 	DIR *ds;
 	struct dirent *de;
 	char *fullpath;
-	ds=opendir(dir);	
+	ds=opendir(dir);
 	if (ds==NULL){
 		ms_message("Cannot open directory %s: %s",dir,strerror(errno));
 		return -1;
@@ -358,7 +377,7 @@ int ms_load_plugins(const char *dir){
 			void *handle;
 			fullpath=ms_strdup_printf("%s/%s",dir,de->d_name);
 			ms_message("Loading plugin %s...",fullpath);
-			
+
 			if ( (handle=dlopen(fullpath,RTLD_NOW))==NULL){
 				ms_warning("Fail to load plugin %s : %s",fullpath,dlerror());
 			}else {
@@ -409,7 +428,7 @@ int ms_load_plugins(const char *dir){
 void ms_unload_plugins(){
 #if defined(WIN32) && !defined(_WIN32_WCE)
 	 MSList *elem;
-	
+
 	for(elem=ms_plugins_loaded_list;elem!=NULL;elem=elem->next)
 	{
 		HINSTANCE handle=(HINSTANCE )elem->data;
@@ -456,7 +475,7 @@ extern MSSndCardDesc aq_card_desc;
 extern MSSndCardDesc pulse_card_desc;
 #endif
 
-#ifdef __MACIOUNIT_ENABLED__
+#if TARGET_OS_IPHONE
 extern MSSndCardDesc au_card_desc;
 #endif
 
@@ -486,16 +505,17 @@ static MSSndCardDesc * ms_snd_card_descs[]={
 #ifdef __MACSND_ENABLED__
 	&ca_card_desc,
 #endif
-#ifdef __MAC_AQ_ENABLED__
-	&aq_card_desc,
-#endif
+
 #ifdef __PULSEAUDIO_ENABLED__
 	&pulse_card_desc,
 #endif
 
-#ifdef __MACIOUNIT_ENABLED__
+#if TARGET_OS_IPHONE
 	&au_card_desc,
 #endif
+#ifdef __MAC_AQ_ENABLED__
+	&aq_card_desc,
+#endif
 #ifdef ANDROID
 	&msandroid_sound_card_desc,
 #endif
@@ -524,18 +544,22 @@ extern MSWebCamDesc ms_directx_cam_desc;
 extern MSWebCamDesc ms_dshow_cam_desc;
 #endif
 
-#ifdef __APPLE__
+#if TARGET_OS_MAC && !TARGET_OS_IPHONE
 extern MSWebCamDesc ms_v4m_cam_desc;
 #endif
 
-#if !defined(NO_FFMPEG)
 extern MSWebCamDesc static_image_desc;
+
+#if !defined(NO_FFMPEG)
 extern MSWebCamDesc mire_desc;
 #endif
 #ifdef ANDROID
 extern MSWebCamDesc ms_android_video_capture_desc;
 #endif
 
+#if TARGET_OS_IPHONE && !TARGET_IPHONE_SIMULATOR
+extern MSWebCamDesc ms_v4ios_cam_desc;
+#endif
 static MSWebCamDesc * ms_web_cam_descs[]={
 #ifdef HAVE_LINUX_VIDEODEV2_H
 	&v4l2_card_desc,
@@ -546,20 +570,22 @@ static MSWebCamDesc * ms_web_cam_descs[]={
 #if defined(WIN32) && defined(HAVE_VFW)
 	&ms_vfw_cam_desc,
 #endif
-#if defined(__MINGW32__) || defined (HAVE_DIRECTSHOW) 
+#if defined(__MINGW32__) || defined (HAVE_DIRECTSHOW)
 	&ms_dshow_cam_desc,
 #endif
-#ifdef __APPLE__
+#if TARGET_OS_MAC && !TARGET_OS_IPHONE
 	&ms_v4m_cam_desc,
 #endif
 #if defined (ANDROID)
 	&ms_android_video_capture_desc,
 #endif
+#if TARGET_OS_IPHONE &&  !TARGET_IPHONE_SIMULATOR
+	&ms_v4ios_cam_desc,
+#endif
 #if !defined(NO_FFMPEG)
 	&mire_desc,
-	&static_image_desc,
 #endif
-
+	&static_image_desc,
 	NULL
 };
 
@@ -585,6 +611,10 @@ void ms_init(){
 	int i;
 	MSSndCardManager *cm;
 
+#if defined(ENABLE_NLS)
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif
+
 #if !defined(_WIN32_WCE)
 	if (getenv("MEDIASTREAMER_DEBUG")!=NULL){
 		ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
@@ -594,7 +624,7 @@ void ms_init(){
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	ortp_set_log_handler(ms_android_log_handler);
 #endif
-	ms_message("Registering all filters...");
+	ms_message("Mediastreamer2 " MEDIASTREAMER_VERSION " (git: " GIT_VERSION ") starting.");
 	/* register builtin MSFilter's */
 	for (i=0;ms_filter_descs[i]!=NULL;i++){
 		ms_filter_register(ms_filter_descs[i]);
@@ -620,13 +650,20 @@ void ms_init(){
 #endif
 #endif
 #ifdef PACKAGE_PLUGINS_DIR
-	ms_message("Loading plugins");
+	ms_message("Loading ms plugins fron [%s]",PACKAGE_PLUGINS_DIR);
 	ms_load_plugins(PACKAGE_PLUGINS_DIR);
 #endif
 
 #if defined(ANDROID) && defined (VIDEO_ENABLED)
-	libmsandroiddisplay_init();
+	if (1) {
+		libmsandroidopengldisplay_init();
+	} else {
+		if (!libmsandroiddisplay_init()) {
+			libmsandroiddisplaybad_init();
+		}
+	}
 #endif
+
 	ms_message("ms_init() done");
 }
 
@@ -683,6 +720,7 @@ void ms_set_payload_max_size(int size){
 }
 
 extern void _android_key_cleanup(void*);
+
 void ms_thread_exit(void* ref_val) {
 #ifdef ANDROID
 	// due to a bug in old Bionic version
@@ -694,4 +732,100 @@ void ms_thread_exit(void* ref_val) {
 }
 
 
+#ifdef __MACH__
+#include <sys/types.h>
+#include <sys/timeb.h>
+#endif
 
+void ms_get_cur_time(MSTimeSpec *ret){
+#if defined(_WIN32_WCE) || defined(WIN32)
+	DWORD timemillis;
+#	if defined(_WIN32_WCE)
+	timemillis=GetTickCount();
+#	else
+	timemillis=timeGetTime();
+#	endif
+	ret->tv_sec=timemillis/1000;
+	ret->tv_nsec=(timemillis%1000)*1000000LL;
+#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	ret->tv_sec=tv.tv_sec;
+	ret->tv_nsec=tv.tv_usec*1000LL;
+#elif defined(__MACH__)
+	struct timeb time_val;
+
+	ftime (&time_val);
+	ret->tv_sec = time_val.time;
+	ret->tv_nsec = time_val.millitm * 1000000LL;
+#else
+	struct timespec ts;
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+		ms_fatal("clock_gettime() doesn't work: %s",strerror(errno));
+	}
+	ret->tv_sec=ts.tv_sec;
+	ret->tv_nsec=ts.tv_nsec;
+#endif
+}
+
+struct _MSConcealerContext {
+	int64_t sample_time;
+	int64_t plc_start_time;
+	unsigned long total_number_for_plc;
+	unsigned int max_plc_time;
+};
+
+/*** plc context begin***/
+unsigned long ms_concealer_context_get_total_number_of_plc(MSConcealerContext* obj) {
+	return obj->total_number_for_plc;
+}
+
+MSConcealerContext* ms_concealer_context_new(unsigned int max_plc_time){
+	MSConcealerContext *obj=(MSConcealerContext *) ms_new(MSConcealerContext,1);	
+	obj->sample_time=-1;
+	obj->plc_start_time=-1;
+	obj->total_number_for_plc=0;
+	obj->max_plc_time=max_plc_time;
+	return obj;
+}
+
+void ms_concealer_context_destroy(MSConcealerContext* context) {
+	ms_free(context);
+}
+
+int ms_concealer_inc_sample_time(MSConcealerContext* obj, uint64_t current_time, int time_increment, int got_packet){
+	int plc_duration=0;
+	if (obj->sample_time==-1){
+		obj->sample_time=(int64_t)current_time;
+	}
+	obj->sample_time+=time_increment;
+	if (obj->plc_start_time!=-1 && got_packet){
+		plc_duration=current_time-obj->plc_start_time;
+		obj->plc_start_time=-1;
+		if (plc_duration>obj->max_plc_time) plc_duration=obj->max_plc_time;
+	}
+	return plc_duration;
+}
+
+unsigned int ms_concealer_context_is_concealement_required(MSConcealerContext* obj,uint64_t current_time) {
+	
+	if(obj->sample_time == -1) return 0; /*no valid value*/
+	
+	if (obj->sample_time < current_time){
+		int plc_duration;
+		if (obj->plc_start_time==-1)
+			obj->plc_start_time=obj->sample_time;
+		plc_duration=current_time-obj->plc_start_time;
+		if (plc_duration<obj->max_plc_time) {
+			obj->total_number_for_plc++;
+			return 1;
+		}else{
+			/*reset sample time, so that we don't do PLC anymore and can resync properly when the stream restarts*/
+			obj->sample_time=-1;
+			return 0;
+		}
+	}
+	return 0;
+}
+
+/*** plc context end***/
